@@ -1,28 +1,26 @@
-#!/usr/bin/python
-# -*- coding: utf-8 -*-
-"""
-Contains functions to calculate confidence intervals.
-"""
+"""Contains functions to calculate confidence intervals."""
 from __future__ import print_function
+
+from collections import OrderedDict
 from warnings import warn
+
 import numpy as np
-from scipy.stats import f
 from scipy.optimize import brentq
+from scipy.special import erf
+from scipy.stats import f
+
 from .minimizer import MinimizerException
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
-
-CONF_ERR_GEN    = 'Cannot determine Confidence Intervals'
+CONF_ERR_GEN = 'Cannot determine Confidence Intervals'
 CONF_ERR_STDERR = '%s without sensible uncertainty estimates' % CONF_ERR_GEN
-CONF_ERR_NVARS  = '%s with < 2 variables' % CONF_ERR_GEN
+CONF_ERR_NVARS = '%s with < 2 variables' % CONF_ERR_GEN
+
 
 def f_compare(ndata, nparas, new_chi, best_chi, nfix=1.):
-    """
-    Returns the probalitiy for two given parameter sets.
+    """Return the probalitiy for two given parameter sets.
+
     nfix is the number of fixed parameters.
+
     """
     nparas = nparas + nfix
     nfree = ndata - nparas
@@ -32,7 +30,7 @@ def f_compare(ndata, nparas, new_chi, best_chi, nfix=1.):
 
 
 def copy_vals(params):
-    """Saves the values and stderrs of params in temporay dict"""
+    """Save the values and stderrs of params in temporary dict."""
     tmp_params = {}
     for para_key in params:
         tmp_params[para_key] = (params[para_key].value,
@@ -41,21 +39,20 @@ def copy_vals(params):
 
 
 def restore_vals(tmp_params, params):
-    """Restores values and stderrs of params in temporay dict"""
+    """Restore values and stderrs of params from a temporary dict."""
     for para_key in params:
         params[para_key].value, params[para_key].stderr = tmp_params[para_key]
 
 
-def conf_interval(minimizer, result, p_names=None, sigmas=(0.674, 0.95, 0.997),
+def conf_interval(minimizer, result, p_names=None, sigmas=(1, 2, 3),
                   trace=False, maxiter=200, verbose=False, prob_func=None):
-    """Calculates the confidence interval for parameters
-    from the given a MinimizerResult, output from minimize.
+    """Calculate the confidence interval for parameters.
 
     The parameter for which the ci is calculated will be varied, while
     the remaining parameters are re-optimized for minimizing chi-square.
     The resulting chi-square is used  to calculate the probability with
-    a given statistic e.g. F-statistic. This function uses a 1d-rootfinder
-    from scipy to find the values resulting in the searched confidence
+    a given statistic (e.g., F-test). This function uses a 1d-rootfinder
+    from SciPy to find the values resulting in the searched confidence
     region.
 
     Parameters
@@ -68,37 +65,44 @@ def conf_interval(minimizer, result, p_names=None, sigmas=(0.674, 0.95, 0.997),
         Names of the parameters for which the ci is calculated. If None,
         the ci is calculated for every parameter.
     sigmas : list, optional
-        The probabilities (1-alpha) to find. Default is 1,2 and 3-sigma.
+        The sigma-levels to find. Default is [1, 2, 3]. See Note below.
     trace : bool, optional
-        Defaults to False, if true, each result of a probability calculation
-        is saved along with the parameter. This can be used to plot so
-        called "profile traces".
-    maxiter : int
+        Defaults to False, if True, each result of a probability calculation
+        is saved along with the parameter. This can be used to plot so-called
+        "profile traces".
+    maxiter : int, optional
         Maximum of iteration to find an upper limit. Default is 200.
-    prob_func : ``None`` or callable
+    verbose: bool, optional
+        Print extra debuging information. Default is False.
+    prob_func : None or callable, optional
         Function to calculate the probability from the optimized chi-square.
-        Default (``None``) uses built-in f_compare (F test).
-    verbose: bool
-        print extra debuging information. Default is ``False``.
-
+        Default is None and uses built-in f_compare (F-test).
 
     Returns
     -------
     output : dict
-        A dict, which contains a list of (sigma, vals)-tuples for each name.
-    trace_dict : dict
-        Only if trace is set true. Is a dict, the key is the parameter which
+        A dictionary that contains a list of (sigma, vals)-tuples for each name.
+    trace_dict : dict, optional
+        Only if trace is True. Is a dict, the key is the parameter which
         was fixed. The values are again a dict with the names as keys, but with
         an additional key 'prob'. Each contains an array of the corresponding
         values.
 
+    Note
+    -----
+    The values for `sigma` are taken as the number of standard deviations for
+    a normal distribution and converted to probabilities. That is, the default
+    ``sigma=(1, 2, 3)`` will use probabilities of 0.6827, 0.9545, and 0.9973.
+    If any of the sigma values is less than 1, that will be interpreted as a
+    probability. That is, a value of 1 and 0.6827 will give the same results,
+    within precision.
+
     See also
     --------
     conf_interval2d
 
     Examples
     --------
-
     >>> from lmfit.printfuncs import *
     >>> mini = minimize(some_func, params)
     >>> mini.leastsq()
@@ -111,12 +115,13 @@ def conf_interval(minimizer, result, p_names=None, sigmas=(0.674, 0.95, 0.997),
 
     Now with quantiles for the sigmas and using the trace.
 
-    >>> ci, trace = conf_interval(mini, sigmas=(0.25, 0.5, 0.75, 0.999), trace=True)
+    >>> ci, trace = conf_interval(mini, sigmas=(0.5, 1, 2, 3), trace=True)
     >>> fixed = trace['para1']['para1']
     >>> free = trace['para1']['not_para1']
     >>> prob = trace['para1']['prob']
 
-    This makes it possible to plot the dependence between free and fixed.
+    This makes it possible to plot the dependence between free and fixed
+    parameters.
 
     """
     ci = ConfidenceInterval(minimizer, result, p_names, prob_func, sigmas,
@@ -128,7 +133,7 @@ def conf_interval(minimizer, result, p_names=None, sigmas=(0.674, 0.95, 0.997),
 
 
 def map_trace_to_names(trace, params):
-    "maps trace to param names"
+    """Map trace to parameter names."""
     out = {}
     allnames = list(params.keys()) + ['prob']
     for name in trace.keys():
@@ -141,15 +146,12 @@ def map_trace_to_names(trace, params):
 
 
 class ConfidenceInterval(object):
-    """
-    Class used to calculate the confidence interval.
-    """
-    def __init__(self, minimizer, result, p_names=None, prob_func=None,
-                 sigmas=(0.674, 0.95, 0.997), trace=False, verbose=False,
-                 maxiter=50):
-        """
+    """Class used to calculate the confidence interval."""
 
-        """
+    def __init__(self, minimizer, result, p_names=None, prob_func=None,
+                 sigmas=(1, 2, 3), trace=False, verbose=False,
+                 maxiter=50):
+        """TODO: docstring in public method."""
         self.verbose = verbose
         self.minimizer = minimizer
         self.result = result
@@ -188,11 +190,16 @@ class ConfidenceInterval(object):
 
         self.sigmas = list(sigmas)
         self.sigmas.sort()
+        self.probs = []
+        for sigma in self.sigmas:
+            if sigma < 1:
+                prob = sigma
+            else:
+                prob = erf(sigma/np.sqrt(2))
+            self.probs.append(prob)
 
     def calc_all_ci(self):
-        """
-        Calculates all cis.
-        """
+        """Calculate all confidence intervals."""
         out = OrderedDict()
 
         for p in self.p_names:
@@ -206,15 +213,15 @@ class ConfidenceInterval(object):
         return out
 
     def calc_ci(self, para, direction):
-        """
-        Calculate the ci for a single parameter for a single direction.
-        Direction is either positive or negative 1.
-        """
+        """Calculate the ci for a single parameter for a single direction.
 
+        Direction is either positive or negative 1.
+
+        """
         if isinstance(para, str):
             para = self.params[para]
 
-        #function used to calculate the pro
+        # function used to calculate the pro
         calc_prob = lambda val, prob: self.calc_prob(para, val, prob)
         if self.trace:
             x = [i.value for i in self.params.values()]
@@ -226,7 +233,7 @@ class ConfidenceInterval(object):
         ret = []
         orig_warn_settings = np.geterr()
         np.seterr(all='ignore')
-        for prob in self.sigmas:
+        for prob in self.probs:
             if prob > max_prob:
                 ret.append((prob, direction*np.inf))
                 continue
@@ -252,17 +259,16 @@ class ConfidenceInterval(object):
         return ret
 
     def reset_vals(self):
+        """TODO: add method docstring."""
         restore_vals(self.org, self.params)
 
     def find_limit(self, para, direction):
-        """
-        For given para, search a value so that prob(val) > sigmas.
-        """
+        """For given para, search a value so that prob(val) > sigmas."""
         if self.verbose:
             print('Calculating CI for ' + para.name)
         self.reset_vals()
 
-        #starting steps:
+        # starting steps:
         if para.stderr > 0 and para.stderr < abs(para.value):
             step = para.stderr
         else:
@@ -274,7 +280,7 @@ class ConfidenceInterval(object):
         limit = start_val
         i = 0
 
-        while old_prob < max(self.sigmas):
+        while old_prob < max(self.probs):
             i = i + 1
             limit += step * direction
 
@@ -285,13 +291,15 @@ class ConfidenceInterval(object):
             # Check convergence.
             if i > self.maxiter:
                 errmsg = "Warning, maxiter={0} reached".format(self.maxiter)
-                errmsg += "and prob({0}={1}) = {2} < max(sigmas).".format(para.name, limit, new_prob)
+                errmsg += ("and prob({0}={1}) = {2} < "
+                           "max(sigmas).".format(para.name, limit, new_prob))
                 warn(errmsg)
                 break
 
             if rel_change < self.min_rel_change:
                 errmsg = "Warning, rel_change={0} < 0.01 ".format(rel_change)
-                errmsg += " at iteration {3} and prob({0}={1}) = {2} < max(sigmas).".format(para.name, limit, new_prob, i)
+                errmsg += (" at iteration {3} and prob({0}={1}) = {2} < max"
+                           "(sigmas).".format(para.name, limit, new_prob, i))
                 warn(errmsg)
                 break
 
@@ -300,7 +308,7 @@ class ConfidenceInterval(object):
         return limit, new_prob
 
     def calc_prob(self, para, val, offset=0., restore=False):
-        """Returns the probability for given Value."""
+        """Calculate the probability for given value."""
         if restore:
             restore_vals(self.org, self.params)
         para.value = val
@@ -317,11 +325,12 @@ class ConfidenceInterval(object):
         self.params[para.name] = save_para
         return prob - offset
 
+
 def conf_interval2d(minimizer, result, x_name, y_name, nx=10, ny=10,
                     limits=None, prob_func=None):
-    r"""Calculates confidence regions for two fixed parameters.
+    r"""Calculate confidence regions for two fixed parameters.
 
-    The method is explained in *conf_interval*: here we are fixing
+    The method itself is explained in *conf_interval*: here we are fixing
     two parameters.
 
     Parameters
@@ -330,38 +339,37 @@ def conf_interval2d(minimizer, result, x_name, y_name, nx=10, ny=10,
         The minimizer to use, holding objective function.
     result : MinimizerResult
         The result of running minimize().
-    x_name : string
+    x_name : str
         The name of the parameter which will be the x direction.
-    y_name : string
+    y_name : str
         The name of the parameter which will be the y direction.
-    nx, ny : ints, optional
-        Number of points.
-    limits : tuple: optional
+    nx : int, optional
+        Number of points in the x direction.
+    ny : int, optional
+        Number of points in the y direction.
+    limits : tuple, optional
         Should have the form ((x_upper, x_lower),(y_upper, y_lower)). If not
         given, the default is 5 std-errs in each direction.
+    prob_func : None or callable, optional
+        Function to calculate the probability from the optimized chi-square.
+        Default is None and uses built-in f_compare (F-test).
 
     Returns
     -------
-    x : (nx)-array
-        x-coordinates
-    y : (ny)-array
-        y-coordinates
-    grid : (nx,ny)-array
-        grid contains the calculated probabilities.
+    x : numpy.ndarray
+        X-coordinates (same shape as nx).
+    y : numpy.ndarray
+        Y-coordinates (same shape as ny).
+    grid : numpy.ndarray
+        Grid containing the calculated probabilities (with shape (nx, ny)).
 
     Examples
     --------
-
     >>> mini = Minimizer(some_func, params)
     >>> result = mini.leastsq()
     >>> x, y, gr = conf_interval2d(mini, result, 'para1','para2')
     >>> plt.contour(x,y,gr)
 
-    Other Parameters
-    ----------------
-    prob_func : ``None`` or callable
-        Function to calculate the probability from the optimized chi-square.
-        Default (``None``) uses built-in f_compare (F test).
     """
     # used to detect that .leastsq() has run!
     params = result.params
@@ -376,10 +384,8 @@ def conf_interval2d(minimizer, result, x_name, y_name, nx=10, ny=10,
     y = params[y_name]
 
     if limits is None:
-        (x_upper, x_lower) = (x.value + 5 * x.stderr, x.value - 5
-                                                      * x.stderr)
-        (y_upper, y_lower) = (y.value + 5 * y.stderr, y.value - 5
-                                                      * y.stderr)
+        (x_upper, x_lower) = (x.value + 5 * x.stderr, x.value - 5 * x.stderr)
+        (y_upper, y_lower) = (y.value + 5 * y.stderr, y.value - 5 * y.stderr)
     elif len(limits) == 2:
         (x_upper, x_lower) = limits[0]
         (y_upper, y_lower) = limits[1]
@@ -392,6 +398,7 @@ def conf_interval2d(minimizer, result, x_name, y_name, nx=10, ny=10,
     y.vary = False
 
     def calc_prob(vals, restore=False):
+        """Calculate the probability."""
         if restore:
             restore_vals(org, result.params)
         x.value = vals[0]
