@@ -1,46 +1,42 @@
-"""
-Parameter class
-"""
+"""Parameter class."""
 from __future__ import division
-import json
-from copy import deepcopy
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
 
-from numpy import array, arcsin, cos, sin, sqrt, inf, nan, isfinite
+from collections import OrderedDict
+from copy import deepcopy
+import json
+
+from numpy import arcsin, array, cos, inf, isfinite, nan, sin, sqrt
+
 from . import uncertainties
 from .asteval import Interpreter
 from .astutils import get_ast_names, valid_symbol_name
 
 
 def check_ast_errors(expr_eval):
-    """check for errors derived from asteval"""
+    """Check for errors derived from asteval."""
     if len(expr_eval.error) > 0:
         expr_eval.raise_exception(None)
 
 
 def isclose(x, y, rtol=1e-5, atol=1e-8):
-    """
-    The truth whether two numbers are the same, within an absolute and
-    relative tolerance.
+    """Check whether two numbers are the same within a tolerance.
 
-    i.e. abs(`x` - `y`) <= (`atol` + `rtol` * absolute(`y`))
+    abs(`x` - `y`) <= (`atol` + `rtol` * abs(`y`))
 
     Parameters
     ----------
     x, y : float
-        Input values
-    rtol : float
-        The relative tolerance parameter (see Notes).
-    atol : float
-        The absolute tolerance parameter (see Notes).
+        Input values.
+    rtol : float, optional
+        The relative tolerance parameter.
+    atol : float, optional
+        The absolute tolerance parameter.
 
     Returns
     -------
-    y : bool
-        Are `x` and `x` are equal within tolerance?
+    bool
+        True if `x` and `y` are the same within tolerance, otherwise False.
+
     """
     def within_tol(x, y, atol, rtol):
         return abs(x - y) <= atol + rtol * abs(y)
@@ -58,21 +54,38 @@ def isclose(x, y, rtol=1e-5, atol=1e-8):
 
 
 class Parameters(OrderedDict):
+    """An ordered dictionary of all the Parameter objects required to
+    specify a fit model. All minimization and Model fitting routines in
+    lmfit will use exactly one Parameters object, typically given as the
+    first argument to the objective function.
+
+    All keys of a Parameters() instance must be strings and valid Python
+    symbol names, so that the name must match ``[a-z_][a-z0-9_]*`` and
+    cannot be a Python reserved word.
+
+    All values of a Parameters() instance must be Parameter objects.
+
+    A Parameters() instance includes an asteval interpreter used for
+    evaluation of constrained Parameters.
+
+    Parameters() support copying and pickling, and have methods to convert
+    to and from serializations using json strings.
+
     """
-    A dictionary of all the Parameters required to specify a fit model.
 
-    All keys must be strings, and valid Python symbol names, and all values
-    must be Parameters.
-
-    Custom methods:
-    ---------------
-
-    add()
-    add_many()
-    dumps() / dump()
-    loads() / load()
-    """
     def __init__(self, asteval=None, *args, **kwds):
+        """
+        Arguments
+        ---------
+        asteval : :class:`asteval.Interpreter`, optional
+            Instance of the asteval Interpreter to use for constraint
+            expressions. If None, a new interpreter will be created.
+        *args : optional
+            Arguments.
+        **kwds : optional
+            Keyword arguments.
+
+        """
         super(Parameters, self).__init__(self)
         self._asteval = asteval
 
@@ -81,23 +94,21 @@ class Parameters(OrderedDict):
         self.update(*args, **kwds)
 
     def copy(self):
-        """Parameters.copy() should always be a deepcopy"""
+        """Parameters.copy() should always be a deepcopy."""
         return self.__deepcopy__(None)
 
-    def __copy__(self, memo):
-        """Parameters.copy() should always be a deepcopy"""
-        self.__deepcopy__(memo)
+    def __copy__(self):
+        """Parameters.copy() should always be a deepcopy."""
+        return self.__deepcopy__(None)
 
     def __deepcopy__(self, memo):
-        """Parameters deepcopy needs to make sure that
-        asteval is available and that all individula
-        parameter objects are copied"""
+        """Parameters.deepcopy() needs to make sure that asteval is available
+        and that all individual Parameter objects are copied."""
         _pars = Parameters(asteval=None)
 
         # find the symbols that were added by users, not during construction
-        sym_unique = self._asteval.user_defined_symbols()
-        unique_symbols = {key: deepcopy(self._asteval.symtable[key], memo)
-                              for key in sym_unique}
+        unique_symbols = {key: self._asteval.symtable[key]
+                          for key in self._asteval.user_defined_symbols()}
         _pars._asteval.symtable.update(unique_symbols)
 
         # we're just about to add a lot of Parameter objects to the newly
@@ -109,10 +120,12 @@ class Parameters(OrderedDict):
                                   min=par.min,
                                   max=par.max)
                 param.vary = par.vary
+                param.brute_step = par.brute_step
                 param.stderr = par.stderr
                 param.correl = par.correl
                 param.init_value = par.init_value
                 param.expr = par.expr
+                param.user_data = par.user_data
                 parameter_list.append(param)
 
         _pars.add_many(*parameter_list)
@@ -120,6 +133,7 @@ class Parameters(OrderedDict):
         return _pars
 
     def __setitem__(self, key, par):
+        """TODO: add magic method docstring."""
         if key not in self:
             if not valid_symbol_name(key):
                 raise KeyError("'%s' is not a valid Parameters name" % key)
@@ -131,9 +145,7 @@ class Parameters(OrderedDict):
         self._asteval.symtable[key] = par.value
 
     def __add__(self, other):
-        """
-        Add Parameters objects
-        """
+        """Add Parameters objects."""
         if not isinstance(other, Parameters):
             raise ValueError("'%s' is not a Parameters object" % other)
         out = deepcopy(self)
@@ -142,9 +154,7 @@ class Parameters(OrderedDict):
         return out
 
     def __iadd__(self, other):
-        """
-        Add/assign Parameters objects
-        """
+        """Add/assign Parameters objects."""
         if not isinstance(other, Parameters):
             raise ValueError("'%s' is not a Parameters object" % other)
         params = other.values()
@@ -152,15 +162,11 @@ class Parameters(OrderedDict):
         return self
 
     def __array__(self):
-        """
-        Parameters to array
-        """
+        """Convert Parameters to array."""
         return array([float(k) for k in self.values()])
 
     def __reduce__(self):
-        """
-        Required to pickle a Parameters instance.
-        """
+        """Reduce Parameters instance such that it can be pickled."""
         # make a list of all the parameters
         params = [self[k] for k in self]
 
@@ -173,8 +179,7 @@ class Parameters(OrderedDict):
                                     'params': params}
 
     def __setstate__(self, state):
-        """
-        Unpickle a Parameters instance.
+        """Unpickle a Parameters instance.
 
         Parameters
         ----------
@@ -182,6 +187,7 @@ class Parameters(OrderedDict):
             state['unique_symbols'] is a dictionary containing symbols that
             need to be injected into _asteval.symtable
             state['params'] is a list of Parameter instances to be added
+
         """
         # first update the Interpreter symbol table. This needs to be done
         # first because Parameter's early in the list may depend on later
@@ -197,20 +203,19 @@ class Parameters(OrderedDict):
         self.add_many(*state['params'])
 
     def update_constraints(self):
-        """
-        Update all constrained parameters, checking that dependencies are
-        evaluated as needed.
-        """
+        """Update all constrained parameters, checking that dependencies are
+        evaluated as needed."""
         requires_update = set(name for name, par in self.items()
                               if par._expr is not None)
         updated_tracker = set(requires_update)
 
         def _update_param(name):
-            """
-            Update a parameter value, including setting bounds.
-            For a constrained parameter (one with an expr defined),
-            this first updates (recursively) all parameters on which
-            the parameter depends (using the 'deps' field).
+            """Update a parameter value, including setting bounds.
+
+            For a constrained parameter (one with an `expr` defined),
+            this first updates (recursively) all parameters on which the
+            parameter depends (using the 'deps' field).
+
             """
             par = self.__getitem__(name)
             if par._expr_eval is None:
@@ -225,6 +230,20 @@ class Parameters(OrderedDict):
             _update_param(name)
 
     def pretty_repr(self, oneline=False):
+        """Return a pretty representation of a Parameters class.
+
+        Parameters
+        ----------
+        oneline : bool, optional
+            If True prints a one-line parameters representation (default is
+            False).
+
+        Returns
+        -------
+        s: str
+           Parameters representation.
+
+        """
         if oneline:
             return super(Parameters, self).__repr__()
         s = "Parameters({\n"
@@ -234,23 +253,25 @@ class Parameters(OrderedDict):
         return s
 
     def pretty_print(self, oneline=False, colwidth=8, precision=4, fmt='g',
-                     columns=['value', 'min', 'max', 'stderr', 'vary', 'expr']):
-        """Pretty-print parameters data.
+                     columns=['value', 'min', 'max', 'stderr', 'vary', 'expr',
+                              'brute_step']):
+        """Pretty-print of parameters data.
 
         Parameters
         ----------
-        oneline : boolean
-            If True prints a one-line parameters representation. Default False.
-        colwidth : int
-            column width for all except the first (i.e. name) column.
-        columns : list of strings
-            list of columns names to print. All values must be valid
-            :class:`Parameter` attributes.
-        fmt : string
-            single-char numeric formatter. Valid values: 'f' floating point,
-            'g' floating point and exponential, 'e' exponential.
-        precision : int
-            number of digits to be printed after floating point.
+        oneline : bool, optional
+            If True prints a one-line parameters representation (default is
+            False).
+        colwidth : int, optional
+            Column width for all columns specified in :attr:`columns`.
+        precision : int, optional
+            Number of digits to be printed after floating point.
+        fmt : {'g', 'e', 'f'}, optional
+            Single-character numeric formatter. Valid values are: 'f' floating
+            point, 'g' floating point and exponential, or 'e' exponential.
+        columns : :obj:`list` of :obj:`str`, optional
+            List of :class:`Parameter` attribute names to print.
+
         """
         if oneline:
             print(self.pretty_repr(oneline=oneline))
@@ -262,7 +283,8 @@ class Parameters(OrderedDict):
         print(title.format(*allcols, name_len=name_len, n=colwidth).title())
         numstyle = '{%s:>{n}.{p}{f}}'  # format for numeric columns
         otherstyles = dict(name='{name:<{name_len}} ', stderr='{stderr!s:>{n}}',
-                           vary='{vary!s:>{n}}', expr='{expr!s:>{n}}')
+                           vary='{vary!s:>{n}}', expr='{expr!s:>{n}}',
+                           brute_step='{brute_step!s:>{n}}')
         line = ' '.join([otherstyles.get(k, numstyle % k) for k in allcols])
         for name, values in sorted(self.items()):
             pvalues = {k: getattr(values, k) for k in columns}
@@ -271,53 +293,78 @@ class Parameters(OrderedDict):
             if 'stderr' in columns and pvalues['stderr'] is not None:
                 pvalues['stderr'] = (numstyle % '').format(
                     pvalues['stderr'], n=colwidth, p=precision, f=fmt)
-            print(line.format(name_len=name_len, n=colwidth, p=precision, f=fmt,
-                              **pvalues))
+            elif 'brute_step' in columns and pvalues['brute_step'] is not None:
+                pvalues['brute_step'] = (numstyle % '').format(
+                    pvalues['brute_step'], n=colwidth, p=precision, f=fmt)
+            print(line.format(name_len=name_len, n=colwidth, p=precision,
+                              f=fmt, **pvalues))
 
-    def add(self, name, value=None, vary=True, min=-inf, max=inf, expr=None):
-        """
-        Convenience function for adding a Parameter:
+    def add(self, name, value=None, vary=True, min=-inf, max=inf, expr=None,
+            brute_step=None):
+        """Add a Parameter.
 
-        Example
-        -------
-        p = Parameters()
-        p.add(name, value=XX, ...)
+        Parameters
+        ----------
+        name : str
+            Name of parameter.  Must match ``[a-z_][a-z0-9_]*`` and cannot be
+            a Python reserved word.
+        value : float, optional
+            Numerical Parameter value, typically the *initial value*.
+        vary : bool, optional
+            Whether the Parameter is varied during a fit (default is True).
+        min : float, optional
+            Lower bound for value (default is `-numpy.inf`, no lower bound).
+        max : float, optional
+            Upper bound for value (default is `numpy.inf`, no upper bound).
+        expr : str, optional
+            Mathematical expression used to constrain the value during the fit.
+        brute_step : float, optional
+            Step size for grid points in the `brute` method.
+
+        Examples
+        --------
+        >>> params = Parameters()
+        >>> params.add('xvar', value=0.50, min=0, max=1)
+        >>> params.add('yvar', expr='1.0 - xvar')
+
+        which is equivalent to:
+
+        >>> params = Parameters()
+        >>> params['xvar'] = Parameter(name='xvar', value=0.50, min=0, max=1)
+        >>> params['yvar'] = Parameter(name='yvar', expr='1.0 - xvar')
 
-        is equivalent to:
-        p[name] = Parameter(name=name, value=XX, ....
         """
         if isinstance(name, Parameter):
             self.__setitem__(name.name, name)
         else:
             self.__setitem__(name, Parameter(value=value, name=name, vary=vary,
-                                             min=min, max=max, expr=expr))
+                                             min=min, max=max, expr=expr,
+                                             brute_step=brute_step))
 
     def add_many(self, *parlist):
-        """
-        Convenience function for adding a list of Parameters.
+        """Add many parameters, using a sequence of tuples.
 
         Parameters
         ----------
-        parlist : sequence
-            A sequence of tuples, or a sequence of `Parameter` instances. If it
-            is a sequence of tuples, then each tuple must contain at least the
-            name. The order in each tuple is the following:
+        parlist : :obj:`sequence` of :obj:`tuple` or :class:`Parameter`
+            A sequence of tuples, or a sequence of `Parameter` instances. If
+            it is a sequence of tuples, then each tuple must contain at least
+            the name. The order in each tuple must be `(name, value, vary,
+            min, max, expr, brute_step)`.
 
-                name, value, vary, min, max, expr
+        Examples
+        --------
+        >>>  params = Parameters()
+        # add with tuples: (NAME VALUE VARY MIN  MAX  EXPR  BRUTE_STEP)
+        >>> params.add_many(('amp',   10, True, None, None, None, None),
+        ...                 ('cen',   4, True,  0.0, None, None, None),
+        ...                 ('wid',   1, False, None, None, None, None),
+        ...                 ('frac', 0.5))
+        # add a sequence of Parameters
+        >>> f = Parameter('par_f', 100)
+        >>> g = Parameter('par_g',  2.)
+        >>> params.add_many(f, g)
 
-        Example
-        -------
-        p = Parameters()
-        # add a sequence of tuples
-        p.add_many( (name1, val1, True, None, None, None),
-                    (name2, val2, True,  0.0, None, None),
-                    (name3, val3, False, None, None, None),
-                    (name4, val4))
-
-        # add a sequence of Parameter
-        f = Parameter('name5', val5)
-        g = Parameter('name6', val6)
-        p.add_many(f, g)
         """
         for para in parlist:
             if isinstance(para, Parameter):
@@ -327,42 +374,59 @@ class Parameters(OrderedDict):
                 self.__setitem__(param.name, param)
 
     def valuesdict(self):
-        """
+        """Return an ordered dictionary of parameter values.
+
         Returns
         -------
-        An ordered dictionary of name:value pairs for each Parameter.
-        This is distinct from the Parameters itself, as it has values of
-        the Parameter values, not the full Parameter object.
-        """
+        OrderedDict
+           An ordered dictionary of :attr:`name`::attr:`value` pairs for each
+           Parameter.
 
+        """
         return OrderedDict(((p.name, p.value) for p in self.values()))
 
     def dumps(self, **kws):
-        """represent Parameters as a JSON string.
+        """Represent Parameters as a JSON string.
 
-        all keyword arguments are passed to `json.dumps()`
+        Parameters
+        ----------
+        **kws : optional
+            Keyword arguments that are passed to `json.dumps()`.
 
         Returns
         -------
-        json string representation of Parameters
+        str
+           JSON string representation of Parameters.
 
         See Also
         --------
         dump(), loads(), load(), json.dumps()
+
         """
-        out = [p.__getstate__() for p in self.values()]
-        return json.dumps(out, **kws)
+        params = [p.__getstate__() for p in self.values()]
+        sym_unique = self._asteval.user_defined_symbols()
+        unique_symbols = {key: deepcopy(self._asteval.symtable[key])
+                          for key in sym_unique}
+        return json.dumps({'unique_symbols': unique_symbols,
+                           'params': params}, **kws)
 
     def loads(self, s, **kws):
-        """load Parameters from a JSON string.
+        """Load Parameters from a JSON string.
 
-        current Parameters will be cleared before loading.
-
-        all keyword arguments are passed to `json.loads()`
+        Parameters
+        ----------
+        **kws : optional
+            Keyword arguments that are passed to `json.loads()`.
 
         Returns
         -------
-        None.   Parameters are updated as a side-effect
+        :class:`Parameters`
+           Updated Parameters from the JSON string.
+
+        Notes
+        -----
+        Current Parameters will be cleared before loading the data from the
+        JSON string.
 
         See Also
         --------
@@ -370,99 +434,125 @@ class Parameters(OrderedDict):
 
         """
         self.clear()
-        for parstate in json.loads(s, **kws):
+
+        tmp = json.loads(s, **kws)
+        state = {'unique_symbols': tmp['unique_symbols'],
+                 'params': []}
+        for parstate in tmp['params']:
             _par = Parameter()
             _par.__setstate__(parstate)
-            self.__setitem__(parstate[0], _par)
+            state['params'].append(_par)
+        self.__setstate__(state)
+        return self
 
     def dump(self, fp, **kws):
-        """write JSON representation of Parameters to a file
-        or file-like object (must have a `write()` method).
+        """Write JSON representation of Parameters to a file-like object.
 
-        Arguments
-        ---------
-        fp         open file-like object with `write()` method.
-
-        all keyword arguments are passed to `dumps()`
+        Parameters
+        ----------
+        fp : file-like object
+            An open and ``.write()``-supporting file-like object.
+        **kws : optional
+            Keyword arguments that are passed to `dumps()`.
 
         Returns
         -------
-        return value from `fp.write()`
+        None or int
+            Return value from `fp.write()`. None for Python 2.7 and the
+            number of characters written in Python 3.
 
         See Also
         --------
         dump(), load(), json.dump()
+
         """
         return fp.write(self.dumps(**kws))
 
     def load(self, fp, **kws):
-        """load JSON representation of Parameters from a file
-        or file-like object (must have a `read()` method).
+        """Load JSON representation of Parameters from a file-like object.
 
-        Arguments
-        ---------
-        fp         open file-like object with `read()` method.
-
-        all keyword arguments are passed to `loads()`
+        Parameters
+        ----------
+        fp : file-like object
+            An open and ``.read()``-supporting file-like object.
+        **kws : optional
+            Keyword arguments that are passed to `loads()`.
 
         Returns
         -------
-        None.   Parameters are updated as a side-effect
+        :class:`Parameters`
+           Updated Parameters loaded from `fp`.
 
         See Also
         --------
         dump(), loads(), json.load()
+
         """
         return self.loads(fp.read(), **kws)
 
 
 class Parameter(object):
+    """A Parameter is an object that can be varied in a fit, or one of the
+    controlling variables in a model. It is a central component of lmfit,
+    and all minimization and modeling methods use Parameter objects.
+
+    A Parameter has a `name` attribute, and a scalar floating point
+    `value`.  It also has a `vary` attribute that describes whether the
+    value should be varied during the minimization.  Finite bounds can be
+    placed on the Parameter's value by setting its `min` and/or `max`
+    attributes.  A Parameter can also have its value determined by a
+    mathematical expression of other Parameter values held in the `expr`
+    attrribute.  Additional attributes include `brute_step` used as the step
+    size in a brute-force minimization, and `user_data` reserved
+    exclusively for user's need.
+
+    After a minimization, a Parameter may also gain other attributes,
+    including `stderr` holding the estimated standard error in the
+    Parameter's value, and `correl`, a dictionary of correlation values
+    with other Parameters used in the minimization.
+
     """
-    A Parameter is an object used to define a Fit Model.
-    Attributes
-    ----------
-    name : str
-        Parameter name.
-    value : float
-        The numerical value of the Parameter.
-    vary : bool
-        Whether the Parameter is fixed during a fit.
-    min : float
-        Lower bound for value (None or -inf means no lower bound).
-    max : float
-        Upper bound for value (None or inf means no upper bound).
-    expr : str
-        An expression specifying constraints for the parameter.
-    stderr : float
-        The estimated standard error for the best-fit value.
-    correl : dict
-        Specifies correlation with the other fitted Parameter after a fit.
-        Of the form `{'decay': 0.404, 'phase': -0.020, 'frequency': 0.102}`
-    """
-    def __init__(self, name=None, value=None, vary=True,
-                 min=-inf, max=inf, expr=None):
+
+    def __init__(self, name=None, value=None, vary=True, min=-inf, max=inf,
+                 expr=None, brute_step=None, user_data=None):
         """
         Parameters
         ----------
         name : str, optional
-            Name of the parameter.
+            Name of the Parameter.
         value : float, optional
             Numerical Parameter value.
         vary : bool, optional
-            Whether the Parameter is fixed during a fit.
+            Whether the Parameter is varied during a fit (default is True).
         min : float, optional
-            Lower bound for value (None or -inf means no lower bound).
+            Lower bound for value (default is `-numpy.inf`, no lower bound).
         max : float, optional
-            Upper bound for value (None or inf means no upper bound).
+            Upper bound for value (default is `numpy.inf`, no upper bound).
         expr : str, optional
             Mathematical expression used to constrain the value during the fit.
+        brute_step : float, optional
+            Step size for grid points in the `brute` method.
+        user_data : optional
+            User-definable extra attribute used for a Parameter.
+
+        Attributes
+        ----------
+        stderr : float
+            The estimated standard error for the best-fit value.
+        correl : dict
+            A dictionary of the correlation with the other fitted Parameters
+            of the form::
+
+            `{'decay': 0.404, 'phase': -0.020, 'frequency': 0.102}`
+
         """
         self.name = name
         self._val = value
-        self.user_value = value
+        self.user_data = user_data
         self.init_value = value
         self.min = min
         self.max = max
+        self.brute_step = brute_step
         self.vary = vary
         self._expr = expr
         self._expr_ast = None
@@ -474,39 +564,80 @@ class Parameter(object):
         self.from_internal = lambda val: val
         self._init_bounds()
 
-    def set(self, value=None, vary=None, min=-inf, max=inf, expr=None):
-        """
-        Set or update Parameter attributes.
+    def set(self, value=None, vary=None, min=None, max=None, expr=None,
+            brute_step=None):
+        """Set or update Parameter attributes.
 
         Parameters
         ----------
         value : float, optional
             Numerical Parameter value.
         vary : bool, optional
-            Whether the Parameter is fixed during a fit.
+            Whether the Parameter is varied during a fit.
         min : float, optional
-            Lower bound for value. To remove a lower bound you must use -np.inf
+            Lower bound for value. To remove a lower bound you must use
+            `-numpy.inf`.
         max : float, optional
-            Upper bound for value. To remove an upper bound you must use np.inf
+            Upper bound for value. To remove an upper bound you must use
+            `numpy.inf`.
         expr : str, optional
             Mathematical expression used to constrain the value during the fit.
             To remove a constraint you must supply an empty string.
-        """
+        brute_step : float, optional
+            Step size for grid points in the `brute` method. To remove the
+            step size you must use ``0``.
 
-        self.__set_expression(expr)
+        Notes
+        -----
+        Each argument to `set()` has a default value of `None`, which will
+        leave the current value for the attribute unchanged.  Thus, to lift a
+        lower or upper bound, passing in `None` will not work.  Instead,
+        you must set these to `-numpy.inf` or `numpy.inf`, as with::
+
+            par.set(min=None)        # leaves lower bound unchanged
+            par.set(min=-numpy.inf)  # removes lower bound
+
+        Similarly, to clear an expression, pass a blank string, (not
+        ``None``!) as with::
+
+            par.set(expr=None)  # leaves expression unchanged
+            par.set(expr='')    # removes expression
+
+        Explicitly setting a value or setting `vary=True` will also
+        clear the expression.
+
+        Finally, to clear the brute_step size, pass ``0``, not ``None``::
+
+            par.set(brute_step=None)  # leaves brute_step unchanged
+            par.set(brute_step=0)     # removes brute_step
+
+        """
         if value is not None:
-            self._val = value
+            self.value = value
+            self.__set_expression('')
+
         if vary is not None:
             self.vary = vary
-        if min is None:
-            min = -inf
-        if max is None:
-            max = inf
-        self.min = min
-        self.max = max
+            if vary:
+                self.__set_expression('')
+
+        if min is not None:
+            self.min = min
+
+        if max is not None:
+            self.max = max
+
+        if expr is not None:
+            self.__set_expression(expr)
+
+        if brute_step is not None:
+            if brute_step == 0.0:
+                self.brute_step = None
+            else:
+                self.brute_step = brute_step
 
     def _init_bounds(self):
-        """make sure initial bounds are self-consistent"""
+        """Make sure initial bounds are self-consistent."""
         # _val is None means - infinity.
         if self.max is None:
             self.max = inf
@@ -527,14 +658,16 @@ class Parameter(object):
         self.setup_bounds()
 
     def __getstate__(self):
-        """get state for pickle"""
+        """Get state for pickle."""
         return (self.name, self.value, self.vary, self.expr, self.min,
-                self.max, self.stderr, self.correl, self.init_value)
+                self.max, self.brute_step, self.stderr, self.correl,
+                self.init_value, self.user_data)
 
     def __setstate__(self, state):
-        """set state for pickle"""
-        (self.name, self.value, self.vary, self.expr, self.min,
-         self.max, self.stderr, self.correl, self.init_value) = state
+        """Set state for pickle."""
+        (self.name, self.value, self.vary, self.expr, self.min, self.max,
+         self.brute_step, self.stderr, self.correl, self.init_value,
+         self.user_data) = state
         self._expr_ast = None
         self._expr_eval = None
         self._expr_deps = []
@@ -542,6 +675,7 @@ class Parameter(object):
         self._init_bounds()
 
     def __repr__(self):
+        """Returns printable representation of a Parameter object."""
         s = []
         if self.name is not None:
             s.append("'%s'" % self.name)
@@ -554,16 +688,17 @@ class Parameter(object):
         s.append("bounds=[%s:%s]" % (repr(self.min), repr(self.max)))
         if self._expr is not None:
             s.append("expr='%s'" % self.expr)
+        if self.brute_step is not None:
+            s.append("brute_step=%s" % (self.brute_step))
         return "<Parameter %s>" % ', '.join(s)
 
     def setup_bounds(self):
-        """
-        Set up Minuit-style internal/external parameter transformation
-        of min/max bounds.
+        """Set up Minuit-style internal/external parameter transformation of
+        min/max bounds.
 
         As a side-effect, this also defines the self.from_internal method
         used to re-calculate self.value from the internal value, applying
-        the inverse Minuit-style transformation.  This method should be
+        the inverse Minuit-style transformation. This method should be
         called prior to passing a Parameter to the user-defined objective
         function.
 
@@ -571,9 +706,11 @@ class Parameter(object):
 
         Returns
         -------
-        The internal value for parameter from self.value (which holds
-        the external, user-expected value).   This internal value should
-        actually be used in a fit.
+        _val : float
+            The internal value for parameter from self.value (which holds
+            the external, user-expected value). This internal value should
+            actually be used in a fit.
+
         """
         if self.min is None:
             self.min = -inf
@@ -595,11 +732,19 @@ class Parameter(object):
         return _val
 
     def scale_gradient(self, val):
-        """
+        """Return scaling factor for gradient.
+
+        Parameters
+        ----------
+        val: float
+            Numerical Parameter value.
+
         Returns
         -------
-        scaling factor for gradient the according to Minuit-style
-        transformation.
+        float
+            Scaling factor for gradient the according to Minuit-style
+            transformation.
+
         """
         if self.min == -inf and self.max == inf:
             return 1.0
@@ -611,16 +756,15 @@ class Parameter(object):
             return cos(val) * (self.max - self.min) / 2.0
 
     def _getval(self):
-        """get value, with bounds applied"""
-
+        """Get value, with bounds applied."""
         # Note assignment to self._val has been changed to self.value
         # The self.value property setter makes sure that the
         # _expr_eval.symtable is kept updated.
         # If you just assign to self._val then
         # _expr_eval.symtable[self.name]
         # becomes stale if parameter.expr is not None.
-        if (isinstance(self._val, uncertainties.Variable)
-            and self._val is not nan):
+        if (isinstance(self._val, uncertainties.Variable) and
+                self._val is not nan):
 
             try:
                 self.value = self._val.nominal_value
@@ -638,28 +782,27 @@ class Parameter(object):
                     self.value = self._expr_eval(self._expr_ast)
                     check_ast_errors(self._expr_eval)
 
-        v = self._val
-        if v > self.max:
-            v = self.max
-        if v < self.min:
-            v = self.min
-        self.value = self._val = v
+        if self._val is not None:
+            if self._val > self.max:
+                self._val = self.max
+            elif self._val < self.min:
+                self._val = self.min
+        if self._expr_eval is not None:
+            self._expr_eval.symtable[self.name] = self._val
         return self._val
 
     def set_expr_eval(self, evaluator):
-        """set expression evaluator instance"""
+        """Set expression evaluator instance."""
         self._expr_eval = evaluator
 
     @property
     def value(self):
-        """The numerical value of the Parameter, with bounds applied"""
+        """Return the numerical value of the Parameter, with bounds applied."""
         return self._getval()
 
     @value.setter
     def value(self, val):
-        """
-        Set the numerical Parameter value.
-        """
+        """Set the numerical Parameter value."""
         self._val = val
         if not hasattr(self, '_expr_eval'):
             self._expr_eval = None
@@ -668,16 +811,17 @@ class Parameter(object):
 
     @property
     def expr(self):
-        """
-        The mathematical expression used to constrain the value during the fit.
-        """
+        """Return the mathematical expression used to constrain the value
+        during the fit."""
         return self._expr
 
     @expr.setter
     def expr(self, val):
-        """
-        The mathematical expression used to constrain the value during the fit.
+        """Set the mathematical expression used to constrain the value during
+        the fit.
+
         To remove a constraint you must supply an empty string.
+
         """
         self.__set_expression(val)
 
@@ -750,7 +894,7 @@ class Parameter(object):
         return self._getval() // other
 
     def __divmod__(self, other):
-        """divmod"""
+        """divmod."""
         return divmod(self._getval(), other)
 
     def __mod__(self, other):
@@ -824,6 +968,6 @@ class Parameter(object):
 
 
 def isParameter(x):
-    """Test for Parameter-ness"""
+    """Test for Parameter-ness."""
     return (isinstance(x, Parameter) or
             x.__class__.__name__ == 'Parameter')
