@@ -7,6 +7,7 @@ import numpy as np
 
 from lmfit import Model, Parameter, models
 from lmfit.lineshapes import gaussian
+from lmfit.models import PseudoVoigtModel
 
 def assert_results_close(actual, desired, rtol=1e-03, atol=1e-03,
                          err_msg='', verbose=True):
@@ -134,6 +135,18 @@ class CommonTests(object):
         if hasattr(short_eval, '__len__'):
             self.assertEqual(len(short_eval), 3)
 
+    def test_result_report(self):
+        pars = self.model.make_params(**self.guess())
+        result = self.model.fit(self.data, pars, x=self.x)
+        report = result.fit_report()
+        assert("[[Model]]" in report)
+        assert("[[Variables]]" in report)
+        assert("[[Fit Statistics]]" in report)
+        assert(" # function evals   =" in report)
+        assert(" Akaike " in report)
+        assert(" chi-square " in report)
+
+
     def test_data_alignment(self):
         _skip_if_no_pandas()
         from pandas import Series
@@ -391,6 +404,38 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         for mod in [model1, model2, model3, model4]:
             self.assertTrue(mod in model_total3.components)
 
+
+    def test_eval_components(self):
+        model1 = models.GaussianModel(prefix='g1_')
+        model2 = models.GaussianModel(prefix='g2_')
+        model3 = models.ConstantModel(prefix='bkg_')
+        mod  = model1 + model2 + model3
+        pars = mod.make_params()
+
+        values1 = dict(amplitude=7.10, center=1.1, sigma=2.40)
+        values2 = dict(amplitude=12.2, center=2.5, sigma=0.5)
+        data  = (1.01 + gaussian(x=self.x, **values1) +
+                 gaussian(x=self.x, **values2) + 0.05*self.noise)
+
+        pars['g1_sigma'].set(2)
+        pars['g1_center'].set(1, max=1.5)
+        pars['g1_amplitude'].set(3)
+        pars['g2_sigma'].set(1)
+        pars['g2_center'].set(2.6, min=2.0)
+        pars['g2_amplitude'].set(1)
+        pars['bkg_c'].set(1.88)
+
+        result = mod.fit(data, params=pars, x=self.x)
+
+        self.assertTrue(abs(result.params['g1_amplitude'].value -  7.1) < 1.5)
+        self.assertTrue(abs(result.params['g2_amplitude'].value - 12.2) < 1.5)
+        self.assertTrue(abs(result.params['g1_center'].value    -  1.1) < 0.2)
+        self.assertTrue(abs(result.params['g2_center'].value    -  2.5) < 0.2)
+        self.assertTrue(abs(result.params['bkg_c'].value    -  1.0) < 0.25)
+
+        comps = mod.eval_components(x=self.x)
+        assert('bkg_' in comps)
+
     def test_composite_has_bestvalues(self):
         # test that a composite model has non-empty best_values
         model1 = models.GaussianModel(prefix='g1_')
@@ -473,6 +518,11 @@ class TestUserDefiniedModel(CommonTests, unittest.TestCase):
         self.assertEqual(models[0].param_hints['amp'],
                          models[1].param_hints['amp'])
 
+    def test_param_hint_explicit_value(self):
+        # tests Github Issue 384
+        pmod = PseudoVoigtModel()
+        params = pmod.make_params(sigma=2, fraction=0.77)
+        assert_allclose(params['fraction'].value, 0.77, rtol=0.01)
 
     def test_composite_model_with_expr_constrains(self):
         """Smoke test for composite model fitting with expr constraints.
@@ -585,4 +635,33 @@ class TestComplexConstant(CommonTests, unittest.TestCase):
         self.guess = lambda: dict(re=2,im=2)
         self.model_constructor = models.ComplexConstantModel
         super(TestComplexConstant, self).setUp()
+
+class TestExpression(CommonTests, unittest.TestCase):
+    def setUp(self):
+        self.true_values = lambda: dict(off_c=0.25, amp_c=1.0, x0=2.0)
+        self.guess = lambda: dict(off_c=0.20, amp_c=1.5, x0=2.5)
+        self.expression = "off_c + amp_c * exp(-x/x0)"
+        self.model_constructor = (
+            lambda *args, **kwargs: models.ExpressionModel(self.expression, *args, **kwargs))
+        super(TestExpression, self).setUp()
+
+    def test_composite_with_expression(self):
+        expression_model = models.ExpressionModel("exp(-x/x0)", name='exp')
+        amp_model = models.ConstantModel(prefix='amp_')
+        off_model = models.ConstantModel(prefix='off_', name="off")
+
+        comp_model = off_model + amp_model * expression_model
+
+        x = self.x
+        true_values = self.true_values()
+        data = comp_model.eval(x=x, **true_values) + self.noise
+        # data = 0.25 + 1 * np.exp(-x / 2.)
+
+        params = comp_model.make_params(**self.guess())
+
+        result = comp_model.fit(data, x=x, params=params)
+        assert_results_close(result.values, true_values, rtol=0.01, atol=0.01)
+
+        data_components = comp_model.eval_components(x=x)
+        self.assertIn('exp', data_components)
 #
