@@ -4,10 +4,12 @@ from lmfit import minimize, Parameters, Parameter, report_fit, Minimizer
 from lmfit.minimizer import (SCALAR_METHODS, HAS_EMCEE,
                              MinimizerResult, _lnpost, _nan_policy)
 from lmfit.lineshapes import gaussian
+from lmfit import ufloat
 import numpy as np
 from numpy import pi
 from numpy.testing import (assert_, decorators, assert_raises,
-                           assert_almost_equal, assert_equal)
+                           assert_almost_equal, assert_equal,
+                           assert_allclose)
 import unittest
 import nose
 from nose import SkipTest
@@ -281,6 +283,48 @@ def test_scalar_minimize_has_no_uncertainties():
     assert_(out2.errorbars == False)
 
 
+def test_scalar_minimize_reduce_fcn():
+    # test that the reduce_fcn option for scalar_minimize
+    # gives different and improved results with outliers
+
+    np.random.seed(2)
+    x = np.linspace(0, 10, 101)
+
+    yo = 1.0 + 2.0*np.sin(4*x) * np.exp(-x / 5)
+    y = yo + np.random.normal(size=len(yo), scale=0.250)
+    outliers = np.random.random_integers(int(len(x)/3.0), len(x)-1,
+                                         int(len(x)/12))
+    y[outliers] += 5*np.random.random(len(outliers))
+
+    # define objective function: returns the array to be minimized
+    def objfunc(pars, x, data):
+        decay = pars['decay']
+        offset= pars['offset']
+        omega = pars['omega']
+        amp   = pars['amp']
+        model = offset + amp * np.sin(x*omega) * np.exp(-x/decay)
+        return model - data
+
+    # create a set of Parameters
+    params = Parameters()
+    params.add('offset', 2.0)
+    params.add('omega',  3.3)
+    params.add('amp',    2.5)
+    params.add('decay',  1.0)
+
+    method='L-BFGS-B'
+    out1 = minimize(objfunc, params, args=(x, y), method=method)
+    out2 = minimize(objfunc, params, args=(x, y), method=method,
+                    reduce_fcn='neglogcauchy')
+
+    #print assert all
+    assert_allclose(out1.params['omega'].value, 4.0, rtol=0.01)
+    assert_allclose(out1.params['decay'].value, 7.6, rtol=0.01)
+
+    assert_allclose(out2.params['omega'].value, 4.0, rtol=0.01)
+    assert_allclose(out2.params['decay'].value, 5.8, rtol=0.01)
+
+
 def test_multidimensional_fit_GH205():
     # test that you don't need to flatten the output from the objective
     # function. Tests regression for GH205.
@@ -305,6 +349,25 @@ def test_multidimensional_fit_GH205():
     mini = Minimizer(fcn2min, params, fcn_args=(xv, yv, data))
     res = mini.minimize()
 
+def test_ufloat():
+    """
+    test of ufloat from uncertainties
+    """
+    x = ufloat((1, 0.1))
+    assert_allclose(x.nominal_value, 1.0, rtol=1.e-7)
+    assert_allclose(x.std_dev(),     0.1, rtol=1.e-7)
+
+    y = x*x
+    assert_allclose(y.nominal_value, 1.0, rtol=1.e-7)
+    assert_allclose(y.std_dev(),     0.2, rtol=1.e-7)
+
+    y = x - x
+    assert_allclose(y.nominal_value, 0.0, rtol=1.e-7)
+    assert_allclose(y.std_dev(),     0.0, rtol=1.e-7)
+
+
+
+
 class CommonMinimizerTest(unittest.TestCase):
 
     def setUp(self):
@@ -574,7 +637,38 @@ class CommonMinimizerTest(unittest.TestCase):
         assert_(np.isfinite(out.params['amp'].correl['period']))
 
         # the lnprob array should be the same as the chain size
-        assert_(np.size(out.chain)//4 == np.size(out.lnprob))
+        assert_(np.size(out.chain)//out.nvarys == np.size(out.lnprob))
+
+        # test chain output shapes
+        assert_(out.lnprob.shape == (10, (20-5+1)/2) )
+        assert_(out.chain.shape == (10, (20-5+1)/2, out.nvarys) )
+        assert_(out.flatchain.shape == (10*(20-5+1)/2, out.nvarys))
+
+    def test_emcee_PT_output(self):
+        # test mcmc output when using parallel tempering
+        if not HAS_EMCEE:
+            return True
+        try:
+            from pandas import DataFrame
+        except ImportError:
+            return True
+        out = self.mini.emcee(ntemps=6, nwalkers=10, steps=20, burn=5, thin=2)
+        assert_(isinstance(out, MinimizerResult))
+        assert_(isinstance(out.flatchain, DataFrame))
+
+        # check that we can access the chains via parameter name
+        assert_(out.flatchain['amp'].shape[0] == 80)
+        assert_(out.errorbars is True)
+        assert_(np.isfinite(out.params['amp'].correl['period']))
+
+        # the lnprob array should be the same as the chain size
+        assert_(np.size(out.chain)//out.nvarys == np.size(out.lnprob))
+
+        # test chain output shapes
+        assert_(out.lnprob.shape == (6, 10, (20-5+1)/2) )
+        assert_(out.chain.shape == (6, 10, (20-5+1)/2, out.nvarys) )
+        # Only the 0th temperature is returned
+        assert_(out.flatchain.shape == (10*(20-5+1)/2, out.nvarys))
 
     @decorators.slow
     def test_emcee_float(self):
