@@ -3,7 +3,7 @@ from lmfit import Parameters, Parameter
 from lmfit.parameter import isclose
 from numpy.testing import assert_, assert_almost_equal, assert_equal
 import unittest
-from copy import deepcopy
+from copy import deepcopy, copy
 import numpy as np
 import pickle
 
@@ -41,6 +41,35 @@ class TestParameters(unittest.TestCase):
         assert(p2._asteval.symtable is not None)
         assert((p2['y'].value > 20) and (p2['y'].value < 21))
 
+    def test_copy_function(self):
+        # check copy(Parameters) does not fail
+        p1 = Parameters()
+        p1.add('t', 2.0, min=0.0, max=5.0)
+        p1.add('x', 10.0)
+        p1.add('y', expr='x*t + sqrt(t)/3.0')
+
+        p2 = copy(p1)
+        assert(isinstance(p2, Parameters))
+
+        # change the 'x' value in the original
+        p1['x'].value = 4.0
+
+        assert(p2['x'].value > 9.8)
+        assert(p2['x'].value < 10.2)
+        assert(np.isinf(p2['x'].max) and p2['x'].max > 0)
+
+        assert('t' in p2)
+        assert('y' in p2)
+        assert(p2['t'].max < 6.0)
+
+        assert(np.isinf(p2['x'].min) and p2['x'].min < 0)
+        assert('sqrt(t)' in p2['y'].expr )
+        assert(p2._asteval is not None)
+        assert(p2._asteval.symtable is not None)
+        assert((p2['y'].value > 20) and (p2['y'].value < 21))
+
+        assert(p1['y'].value < 10)
+
 
     def test_deepcopy(self):
         # check that a simple copy works
@@ -138,6 +167,31 @@ class TestParameters(unittest.TestCase):
         pkl = pickle.dumps(p)
         q = pickle.loads(pkl)
 
+
+    def test_set_symtable(self):
+        # test that we use Parameter.set(value=XXX) and have
+        # that new value be used in constraint expressions
+        pars = Parameters()
+        pars.add('x', value=1.0)
+        pars.add('y', expr='x + 1')
+
+        assert_(isclose(pars['y'].value, 2.0))
+        pars['x'].set(value=3.0)
+        assert_(isclose(pars['y'].value, 4.0))
+
+    def test_dumps_loads_parameters(self):
+        # test that we can dumps() and then loads() a Parameters
+        pars = Parameters()
+        pars.add('x', value=1.0)
+        pars.add('y', value=2.0)
+        pars['x'].expr = 'y / 2.0'
+
+        dumps = pars.dumps()
+
+        newpars = Parameters().loads(dumps)
+        newpars['y'].value = 100.0
+        assert_(isclose(newpars['x'].value, 50.0))
+
     def test_isclose(self):
         assert_(isclose(1., 1+1e-5, atol=1e-4, rtol=0))
         assert_(not isclose(1., 1+1e-5, atol=1e-6, rtol=0))
